@@ -5,6 +5,7 @@
 package clearsign
 
 import (
+	"bufio"
 	"bytes"
 	"crypto"
 	"fmt"
@@ -171,6 +172,63 @@ func TestSigningInterop(t *testing.T) {
 	}
 }
 
+func TestHashHeader(t *testing.T) {
+	tests := []struct {
+		name      string
+		hashTypes []crypto.Hash
+		expected  string
+	}{
+		{
+			name: "unique hashes",
+			hashTypes: []crypto.Hash{
+				crypto.SHA256,
+				crypto.SHA512,
+				crypto.SHA3_512,
+			},
+			expected: "Hash: SHA256,SHA512,SHA3-512\n",
+		},
+		{
+			name: "with duplicates",
+			hashTypes: []crypto.Hash{
+				crypto.SHA256,
+				crypto.SHA512,
+				crypto.SHA512,
+				crypto.SHA3_512,
+			},
+			expected: "Hash: SHA256,SHA512,SHA3-512\n",
+		},
+		{
+			name: "with duplicates",
+			hashTypes: []crypto.Hash{
+				crypto.SHA256,
+				crypto.SHA256,
+				crypto.SHA256,
+				crypto.SHA256,
+			},
+			expected: "Hash: SHA256\n",
+		},
+	}
+
+	for _, tc := range tests {
+		t.Run(tc.name, func(t *testing.T) {
+			var buf bytes.Buffer
+			writer := bufio.NewWriter(&buf)
+			if err := writeHashHeader(writer, tc.hashTypes); err != nil {
+				t.Fatalf("unexpected error: %v", err)
+			}
+
+			if err := writer.Flush(); err != nil {
+				t.Fatalf("flush failed: %v", err)
+			}
+
+			actual := buf.String()
+			if actual != tc.expected {
+				t.Errorf("output mismatch:\nExpected: %q\nActual:   %q", tc.expected, actual)
+			}
+		})
+	}
+}
+
 func testMultiSign(t *testing.T, v6 bool) {
 	if testing.Short() {
 		t.Skip("skipping long test in -short mode")
@@ -292,6 +350,12 @@ _o/
 -----BEGIN PGP SIGNED MESSAGE-----
 NotHash: SHA256
 
+_o/
+` + signatureBlock,
+	`
+-----BEGIN PGP SIGNED MESSAGE-----
+Hash: SHA256 Hello world this should not be allowed
+
 _o/
 ` + signatureBlock,
 }
